from odoo import http
from odoo.http import request
import base64
import requests
from werkzeug.datastructures import FileStorage
import logging

_logger = logging.getLogger(__name__)


class RasdFormController(http.Controller):

    def verify_recaptcha(self, recaptcha_response, action_expected="submit"):
        """
        Verify reCAPTCHA v3 with Google.
        """
        secret_key = "6LfX9IwqAAAAAKVJ6umlbsKshCcTCDebLXc5ck2o"  # Replace with your reCAPTCHA secret key
        verify_url = "https://www.google.com/recaptcha/api/siteverify"
        payload = {
            'secret': secret_key,
            'response': recaptcha_response,
        }
        try:
            response = requests.post(verify_url, data=payload, timeout=10)
            result = response.json()
            if result.get('success') and result.get('action') == action_expected and result.get('score', 0) >= 0.5:
                return True
            return False
        except Exception as e:
            # Log error for debugging
            request.env['ir.logging'].sudo().create({
                'name': 'reCAPTCHA Verification',
                'type': 'server',
                'message': f"Error during reCAPTCHA verification: {str(e)}",
                'level': 'error',
            })
            return False

    def send_email_with_identifier(self, email, identifier):
        """
        Sends an email to the reporter with the report identifier.
        """
        try:
            mail_values = {
                'subject': 'Your Rasd Report Identifier',
                'body_html': f"""
                    <p>Dear Reporter,</p>
                    <p>Your report has been successfully submitted.</p>
                    <p><b>Report Identifier:</b> {identifier}</p>
                    <p>Please keep this identifier for future reference.</p>
                    <p>Thank you for using our service.</p>
                """,
                'email_to': email,
                'email_from': request.env.user.company_id.email or 'noreply@example.com',
            }
            mail = request.env['mail.mail'].sudo().create(mail_values)
            mail.send()
            _logger.info("Email sent to %s with identifier: %s", email, identifier)
        except Exception as e:
            _logger.error("Failed to send email to %s: %s", email, e)

    def process_uploads(self, files, res_model, res_id):
        """
        Process and save uploaded files as `ir.attachment` records.
        """
        for file in files:
            if isinstance(file, FileStorage) and file.filename:
                request.env['ir.attachment'].sudo().create({
                    'name': file.filename,
                    'datas': base64.b64encode(file.read()).decode('utf-8'),
                    'res_model': res_model,
                    'res_id': res_id,
                    'type': 'binary',
                })

    @http.route('/rasd-forms', type='http', methods=['GET'], auth='public', website=True, csrf=False)
    def show_submission_form(self, **kwargs):
        """
        Display the submission form for visitors.
        """
        return request.render('rasd_forms.website_rasd_submission_form')

    @http.route('/rasd-form-submit', type='http', methods=['POST'], auth='public', website=True, csrf=False)
    def handle_submission_form(self, **post):
        """
        Handle the submission of the form and create records.
        """
        try:
            # Verify reCAPTCHA with Google
            recaptcha_response = post.get('g-recaptcha-response')
            secret_key = "6LfX9IwqAAAAAKVJ6umlbsKshCcTCDebLXc5ck2o"  # Replace with your reCAPTCHA secret key
            verify_url = "https://www.google.com/recaptcha/api/siteverify"
            payload = {
                'secret': secret_key,
                'response': recaptcha_response,
                'remoteip': request.httprequest.remote_addr,  # Optional
            }
            response = requests.post(verify_url, data=payload)
            result = response.json()

            # Debugging
            _logger.info("reCAPTCHA Validation Response: %s", result)

            # Create the RasdForm record
            rasd_form = request.env['rasd.form'].sudo().create({
                'reporter_name': post.get('reporter_name'),
                'id_number': post.get('id_number'),
                'address': post.get('address'),
                'email': post.get('email'),
                'phone': post.get('phone'),
                'report_type': post.get('report_type'),
                'description': post.get('description'),
                'total_estimated_value': post.get('total_estimated_value'),
            })

            # Handle specific loss type logic
            if rasd_form.report_type == 'personal':
                personal_loss = request.env['personal.loss'].sudo().create({
                    'rasd_form_id': rasd_form.id,
                    'name': post.get('name'),
                    'phone': post.get('phone'),
                    'email': post.get('email'),
                    'property_type': post.get('property_type'),
                    'estimated_value': post.get('personal_estimated_value'),
                    'description': post.get('description'),
                })
                files = request.httprequest.files.getlist('personal_upload')
                self.process_uploads(files, 'personal.loss', personal_loss.id)

            elif rasd_form.report_type == 'institutional':
                institutional_loss = request.env['institutional.loss'].sudo().create({
                    'rasd_form_id': rasd_form.id,
                    'institution_name': post.get('institution_name'),
                    'institution_type': post.get('institution_type'),
                    'contact_person': post.get('contact_person'),
                    'estimated_value': post.get('institution_estimated_value'),
                    'estimated_loss_percentage': post.get('estimated_loss_percentage'),
                    'description': post.get('description'),
                })
                files = request.httprequest.files.getlist('institutional_upload')
                self.process_uploads(files, 'institutional.loss', institutional_loss.id)

            elif rasd_form.report_type == 'infrastructure':
                infrastructural_loss = request.env['infrastructure.loss'].sudo().create({
                    'rasd_form_id': rasd_form.id,
                    'infrastructure_name': post.get('infrastructure_name'),
                    'damage_level': post.get('damage_level'),
                    'infrastructure_type': post.get('infrastructure_type'),
                    'estimated_cost': post.get('estimated_cost'),
                    'description': post.get('description'),
                    'infra_contact_person': post.get('infra_contact_person'),
                    'estimated_loss_percentage': post.get('estimated_loss_percentage'),
                })
                files = request.httprequest.files.getlist('infrastructure_upload')
                self.process_uploads(files, 'infrastructure.loss', infrastructural_loss.id)
          
            # Send email with the report identifier
            if rasd_form.email:
                self.send_email_with_identifier(rasd_form.email, rasd_form.name)

            # Render confirmation page with identifier and email
            return request.render('rasd_forms.rasd_form_confirmation', {
                'identifier': rasd_form.name,
                'email': rasd_form.email
            })            
            
        except Exception as e:
            _logger.error("Error processing form submission: %s", e)
            return request.render('rasd_forms.rasd_form_error', {'error': str(e)})

