from odoo import http
from odoo.http import request
import io
from reportlab.lib.pagesizes import letter
from reportlab.pdfgen import canvas
from reportlab.pdfbase.ttfonts import TTFont
from reportlab.pdfbase import pdfmetrics
import arabic_reshaper
from bidi.algorithm import get_display
import os
import requests
import logging
_logger = logging.getLogger(__name__)

class RasdReportController(http.Controller):
    @http.route('/search-selector', type='http', auth="public", website=True)
    def search_selector(self, **kwargs):
        _logger.info("Search Selector Route Hit")
        return request.render('rasd_forms.search_selector')

    @http.route('/search-by-id', type='http', auth="public", website=True)
    def search_by_id(self, **kwargs):
        return request.render('rasd_forms.search_by_id_form')

    @http.route('/search-by-identifier', type='http', auth="public", website=True)
    def search_by_identifier(self, **kwargs):
        return request.render('rasd_forms.search_by_identifier_form')
    @http.route('/search-reports', type='http', auth="public", methods=['POST'], website=True, csrf=True)
    def search_reports(self, **post):
        recaptcha_response = post.get('g-recaptcha-response')
        secret_key = '6LfX9IwqAAAAAKVJ6umlbsKshCcTCDebLXc5ck2o'
        url = "https://www.google.com/recaptcha/api/siteverify"
        payload = {'secret': secret_key, 'response': recaptcha_response}
        response = requests.post(url, data=payload)
        result = response.json()

        if not result.get('success'):
            error = "Invalid reCAPTCHA. Please try again."
            return request.render('rasd_forms.search_selector', {'error': error})

        # Search logic
        id_number = post.get('id_number')
        report_identifier = post.get('report_identifier')

        domain = []
        if id_number:
            domain.append(('id_number', '=', id_number))
        elif report_identifier:
            domain.append(('name', '=', report_identifier))

        reports = request.env['rasd.form'].sudo().search(domain)

        if reports:
            results = []
            for report in reports:
                results.append({
                    'identifier': report.name,
                    'type': report.report_type,
                    'status': report.status,
                    'description': report.description,
                    'date_reported': report.date_reported.strftime('%Y-%m-%d'),
                    'estimated_value': report.total_estimated_value,
                    'gps_location': report.gps_location or 'N/A',
                    'personal_loss': report.personal_loss_ids or [],
                    'institutional_loss': report.institutional_loss_ids or [],
                    'infrastructural_loss': report.infrastructural_loss_ids or [],
                })
            return request.render('rasd_forms.search_results', {'results': results})
        else:
            return request.render('rasd_forms.no_results', {})

    @http.route('/print-pdf', type='http', auth="public", website=True, csrf=False)
    def print_pdf(self, id_number=None, report_identifier=None, **kwargs):
        # Fetch reports based on the provided parameter
        reports = None
        if id_number:
            reports = request.env['rasd.form'].sudo().search([('id_number', '=', id_number)])
        elif report_identifier:
            reports = request.env['rasd.form'].sudo().search([('name', '=', report_identifier)])
        
        if not reports:
            return request.not_found()


        # Generate PDF content
        pdf_content = self._generate_pdf(reports)
        return request.make_response(pdf_content, [
            ('Content-Type', 'application/pdf'),
            ('Content-Disposition', f'inline; filename={id_number or report_identifier}_reports.pdf')
        ])

    def _generate_pdf(self, reports):
        buffer = io.BytesIO()
        p = canvas.Canvas(buffer, pagesize=letter)

        # Register Arabic font
        font_path = os.path.join(os.path.dirname(__file__), '../static/src/fonts/Arial.ttf')
        pdfmetrics.registerFont(TTFont('Arabic', font_path))

        def draw_arabic_text(text, x, y):
            reshaped_text = arabic_reshaper.reshape(text)
            bidi_text = get_display(reshaped_text)
            p.drawRightString(x, y, bidi_text)

        y_position = 750
        for report in reports:
            # Title
            p.setFont("Arabic", 16)
            draw_arabic_text(f"تقرير رصد: {report.name}", 500, y_position)
            y_position -= 40

            # Basic Report Details
            p.setFont("Arabic", 14)
            draw_arabic_text(f"نوع التقرير: {report.report_type}", 500, y_position)
            y_position -= 20
            draw_arabic_text(f"الحالة: {report.status}", 500, y_position)
            y_position -= 20
            draw_arabic_text(f"تاريخ التقرير: {report.date_reported}", 500, y_position)
            y_position -= 20
            draw_arabic_text(f"القيمة التقديرية: {report.total_estimated_value}", 500, y_position)
            y_position -= 40

            # Personal Loss Details
            if report.personal_loss_ids:
                draw_arabic_text("تفاصيل الخسائر الشخصية:", 500, y_position)
                y_position -= 20
                for personal in report.personal_loss_ids:
                    draw_arabic_text(f"- اسم المتضرر: {personal.name}", 500, y_position)
                    y_position -= 20

            # Institutional Loss Details
            if report.institutional_loss_ids:
                draw_arabic_text("تفاصيل الخسائر المؤسسية:", 500, y_position)
                y_position -= 20
                for institutional in report.institutional_loss_ids:
                    draw_arabic_text(f"- اسم المؤسسة: {institutional.institution_name}", 500, y_position)
                    y_position -= 20

            # Infrastructure Loss Details
            if report.infrastructural_loss_ids:
                draw_arabic_text("تفاصيل خسائر البنية التحتية:", 500, y_position)
                y_position -= 20
                for infrastructure in report.infrastructural_loss_ids:
                    draw_arabic_text(f"- اسم البنية التحتية: {infrastructure.infrastructure_name}", 500, y_position)
                    y_position -= 20

            y_position -= 40  # Space between reports
            if y_position < 100:  # Create a new page if necessary
                p.showPage()
                y_position = 750

        p.save()
        pdf_content = buffer.getvalue()
        buffer.close()
        return pdf_content
