from odoo import api, fields, models
from odoo.exceptions import ValidationError


class RasdForm(models.Model):
    _name = 'rasd.form'
    _description = 'Rasd Central Form'

    # Basic Information
    name = fields.Char(string="Report Identifier", readonly=True, default='New')
    reporter_name = fields.Char(string="Reporter Name", required=True)
    id_number = fields.Char(string="National ID/Passport/Driving License No.")
    address = fields.Text(string="Address")
    email = fields.Char(string="Email Address")
    phone = fields.Char(string="Phone Number")

    # Report Type (Personal, Institutional, Infrastructure)
    report_type = fields.Selection([
        ('personal', 'Personal Loss'),
        ('institutional', 'Institutional Loss'),
        ('infrastructure', 'Infrastructural Loss')
    ], string="Report Type", required=True)

    # Loss Details
    gps_location = fields.Char(string="GPS Location")
    description = fields.Text(string="General Description")
    loss_location_type = fields.Selection([
        ('rural', 'Rural'),
        ('urban', 'Urban')
    ], string="Location Type", default='rural')
    additional_notes = fields.Text(string="Additional Notes") #######
    description = fields.Text(string="General Description")
    date_reported = fields.Date(string="Date Reported", default=fields.Date.today)
    status = fields.Selection([
        ('draft', 'Draft'),
        ('submitted', 'Submitted'),
        ('review', 'Under Review'),
        ('approved', 'Approved')
    ], string="Status", default='draft')
    total_estimated_value = fields.Float(string="Total Estimated Loss Value", default=0.0)

    # Relationships with Loss Types
    personal_loss_ids = fields.One2many('personal.loss', 'rasd_form_id', string="Personal Loss Reports")
    institutional_loss_ids = fields.One2many('institutional.loss', 'rasd_form_id', string="Institutional Loss Reports")
    infrastructural_loss_ids = fields.One2many('infrastructure.loss', 'rasd_form_id', string="Infrastructural Loss Reports")
    @api.model
    def create(self, vals):
        """
        Override the create method to generate a unique identifier for the report.
        """
        if vals.get('name', 'New') == 'New':
            # Get the current year
            year = fields.Date.today().year
            # Get the last sequence number for this year
            sequence = self.env['ir.sequence'].next_by_code('rasd.form') or '0000'
            vals['name'] = f'RSD-{year}-{sequence}'
        return super(RasdForm, self).create(vals)

class PersonalLoss(models.Model):
    _name = 'personal.loss'
    _description = 'Personal Loss Form'

    name = fields.Char(string="The affected person")
    phone = fields.Char(string="Phone Number")
    email = fields.Char(string="Email Address")
    rasd_form_id = fields.Many2one('rasd.form', string="Rasd Form", ondelete='cascade')
    property_type = fields.Selection(
    [
        ('residential', 'Residential Property'),
        ('commercial', 'Commercial Property'),
        ('industrial', 'Industrial Property'),
        ('land', 'Land'),
        ('vehicle', 'Vehicle'),
        ('other', 'Other'),
    ],
    string="Property Type",
    default='residential'
    )

    cause_of_loss = fields.Selection(
    [
        ('natural_disaster', 'Natural Disaster (e.g., Flood, Earthquake)96'),
        ('fire', 'Fire'),
        ('theft', 'Theft'),
        ('flood', 'Flood'),
        ('accident', 'Accident'),
        ('vandalism', 'Vandalism'),
        ('other', 'Other'),
    ],
    string="Cause of Loss",
    default='natural_disaster'
    )
    severity_scale = fields.Selection(
    [
        ('1', '1 - Minimal'),
        ('2', '2'),
        ('3', '3'),
        ('4', '4'),
        ('5', '5 - Moderate'),
        ('6', '6'),
        ('7', '7'),
        ('8', '8'),
        ('9', '9'),
        ('10', '10 - Severe'),
    ],
    string="Severity Scale",
    default='1'
    )
    estimated_value = fields.Float(string="Estimated Loss Value")
    description = fields.Text(string="Description")
    personal_upload = fields.One2many(
        'ir.attachment', 'res_id',
        domain=[('res_model', '=', 'personal.loss')],
        string="Photos"
    )

class InstitutionalLoss(models.Model):
    _name = 'institutional.loss'
    _description = 'Institutional Loss Form'

    rasd_form_id = fields.Many2one('rasd.form', string="Rasd Form", ondelete='cascade')
    institution_name = fields.Char(string="Institution Name", required=True)
    institution_type = fields.Selection(
    [
        ('hospital', 'Hospital'),
        ('school', 'School'),
        ('factory', 'Factory'),
        ('government', 'Government Facility')
    ], string="Institution Type")
    contact_person = fields.Char(string="Contact Person")
    estimated_value = fields.Float(string="Estimated Loss Value")
    description = fields.Text(string="Description")
    staff_affected_count = fields.Integer(string="Number of Staff Affected") ########
    infrastructure_dependency = fields.Selection(
    [
        ('essential', 'Essential (e.g., Hospitals, Emergency Services)'),
        ('major', 'Major (e.g., Highways, Bridges)'),
        ('moderate', 'Moderate (e.g., Local Roads, Secondary Facilities)'),
        ('minor', 'Minor (e.g., Recreational, Non-critical Infrastructure)'),
    ],
    string="Infrastructure Dependency",
    default='essential'
    ) ########
    estimated_loss_percentage = fields.Float(string="Estimated Loss Percentage")
    institutional_upload = fields.One2many(
        'ir.attachment', 'res_id',
        domain=[('res_model', '=', 'institutional.loss')],
        string="Photos"
    )

class InfrastructureLoss(models.Model):
    _name = 'infrastructure.loss'
    _description = 'Infrastructural Loss Form'

    rasd_form_id = fields.Many2one('rasd.form', string="Rasd Form", ondelete='cascade')
    infrastructure_name = fields.Char(string="Infrastructure Name", required=True)
    damage_level = fields.Selection([
        ('minor', 'Minor'),
        ('moderate', 'Moderate'),
        ('severe', 'Severe'),
        ('complete', 'Complete Destruction')
    ], string="Damage Level")
    infrastructure_type = fields.Selection([
        ('road', 'Road'),
        ('bridge', 'Bridge'),
        ('electric_station', 'Electric Station'),
        ('water_facility', 'Water Facility'),
        ('telecom_tower', 'Telecommunication Tower'),
        ('port', 'Port'),
        ('railway', 'Railway'),
        ('airport', 'Airport'),
        ('building', 'Building'),
        ('dam', 'Dam')
    ], string="Infrastructure Type")
    estimated_cost = fields.Float(string="Estimated Repair Cost")
    description = fields.Text(string="Description")
    estimated_loss_percentage = fields.Float(string="Estimated Loss Percentage")
    reconstruction_timeframe = fields.Integer(string="Reconstruction Timeframe (days)") ########
    environmental_impact = fields.Text(string="Environmental Impact")
    infra_contact_person = fields.Char(string="Contact Person") ########
    infrastructure_upload = fields.One2many(
        'ir.attachment', 'res_id',
        domain=[('res_model', '=', 'infrastructure.loss')],
        string="Photos"
    )


